# -*- encoding: utf-8 -*
from django.conf import settings
from django.db import models
from dna.replicate import ReplicateDjangoModel


def every_model(model_dna):
    """Utility function to build every model + the eliolist model."""
    replicator = ReplicateDjangoModel(model_dna)
    replicator.start_replication()


def eliolist_model():
    class ElioList(models.Model):
        parent_model = models.ForeignKey(
            "Thing", on_delete=models.CASCADE, related_name="parent_to_list"
        )
        listed_model = models.ForeignKey(
            "Thing", on_delete=models.CASCADE, related_name="listed_by_parent"
        )
        field_name = models.CharField(max_length=100)

        class Meta:
            db_table = "eliolist"
            app_label = settings.APP_NAME
            unique_together = ["parent_model", "listed_model", "field_name"]

    return ElioList



"""**Usage**

::

    # Show the full scheme of a Class or Property.
    django-admin genome Thing
    django-admin genome description"""
# -*- encoding: utf-8 -*-
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from dna.crispr import Crispr


class Command(BaseCommand):
    """Build a mock file you can use to test with from a Thing."""

    def add_arguments(self, parser):
        """
        :param name: Something to .
        """
        parser.add_argument("name", type=str, help="A thing to examine")
        parser.add_argument(
            "-d",
            "--dependants",
            dest="show_deps",
            action="store_true",
            help="Show dependants.",
        )
        parser.set_defaults(show_deps=False)

    def genome_comment(self, obj):
        comment = obj["rdfs:comment"]
        self.stdout.write(comment)

    def enumerated_genome(self, enumerations):
        if enumerations:
            self.stdout.write("Enumerated:")
            for enum in sorted(enumerations):
                self.stdout.write(f" |- {enum}")

    def subclasses_genome(self, subclasses):
        if subclasses:
            self.stdout.write("Subclasses:")
            for sub in sorted(subclasses):
                self.stdout.write(f" |- {sub}")

    def properties_genome(self, properties):
        if properties:
            self.stdout.write("Properties:")
            for prop_name in sorted(
                [Crispr.property_name(p) for p in properties]
            ):
                self.stdout.write(f" |- {prop_name}")

    def dependants_dependants_print(self, dependants, level):
        if isinstance(dependants, list):
            for sub in dependants:
                self.dependants_dependants_print(sub, level + 1)
        else:
            space = "   " * level
            self.stdout.write(f"{space}|- {dependants}")

    def dependants_genome(self, dependants):
        if dependants:
            self.stdout.write("Dependants:")
            self.dependants_dependants_print(dependants, 0)

    def handle(self, *args, **options):
        name = options["name"]
        show_deps = options["show_deps"]
        crispr = Crispr(
            f"schemaorg/data/releases/{settings.SCHEMA_VERSION}/all-layers.jsonld"
        )

        property_of = None
        class_of = crispr.class_of_by_name(name)
        if not class_of:
            property_of = crispr.property_of_by_name(name)

        if not property_of and not class_of:
            raise CommandError(f"{name} not found.")

        if property_of:
            self.stdout.write(f"Property: {name}")
            self.genome_comment(property_of)
            self.stdout.write("Types:")
            for type in crispr.property_types_of(property_of, crispr.domain):
                self.stdout.write(f" |- {type}")
            self.stdout.write("Classes:")
            for cls in crispr.property_domains_of(property_of, crispr.domain):
                self.stdout.write(f" |- {cls}")

            self.enumerated_genome(
                crispr.enumerations_of_by_url(
                    crispr.property_enumerated(property_of)
                )
            )

        if class_of:
            self.stdout.write(f"Class: {name}")
            self.genome_comment(class_of)
            if crispr.is_primitive_thing(class_of, crispr.domain):
                self.stdout.write(" |- PRIMITIVE TYPE")
            # self.stdout.write(str(class_of))
            self.subclasses_genome(crispr.subclasses_of(class_of))
            self.properties_genome(crispr.properties_of_by_name(name))
            self.enumerated_genome(crispr.enumerations_of_by_name(name))
            if show_deps:
                self.dependants_genome(crispr.descendants_of_by_name(name))



"""Crispr: Process a schema.org jsonld file ready for django models.

This class extracts all the required information to build a set of dynamic
Django models.


**Usage**

::

    from dna.crispr import Crispr
    crispr = Crispr("schemaorg/data/releases/3.9/all-layers.jsonld")"""
# -*- encoding: utf-8 -*
import json


def get_schema(schema_file):
    """Utility to open and load a jsonld file."""
    with open(schema_file) as json_data:
        return json.load(json_data)


class Crispr:
    """Parse a schemaorg jsonld."""

    def __init__(self, schema_path, domain="http://schema.org"):
        self.data = get_schema(schema_path)
        self.domain = domain
        # Separate classes from properties in advance.
        self.classes = [
            cls
            for cls in self.data.get("@graph", [])
            if "rdfs:Class" in self.list_anyway(cls["@type"])
        ]
        self.properties = [
            prp
            for prp in self.data.get("@graph", [])
            if prp["@type"] == "rdf:Property"
        ]
        self.enumerated = [
            obj
            for obj in self.data.get("@graph", [])
            if obj not in self.classes + self.properties
        ]
        self.primitives = [
            prm["@id"]
            for prm in self.classes
            if Crispr.is_primitive_thing(prm, self.domain)
        ] + [
            # Sub classed primitives
            "http://schema.org/Duration",
            "http://schema.org/Float",
            "http://schema.org/Integer",
            "http://schema.org/Quantity",
            "http://schema.org/URL",
        ]

    @staticmethod
    def merge(list1, list2):
        """Merge two lists without duplication."""
        for l in list2:
            if l not in list1:
                list1.append(l)
        return list1

    @staticmethod
    def list_anyway(obj):
        """Some values are lists, some dictionaries. Meh. List anyway."""
        if isinstance(obj, list):
            return obj
        else:
            return [obj]

    @staticmethod
    def is_of(obj, thing_url):
        """This is not the thing you are looking for."""
        return obj.get("@id") == thing_url

    @staticmethod
    def property_types_of(prp, domain):
        """Get all the Types supported by this Property."""
        types = []
        typesof = prp.get(f"{domain}/rangeIncludes", [])
        for range in Crispr.list_anyway(typesof):
            types.append(range["@id"])
        return types

    @staticmethod
    def property_domains_of(prp, domain):
        """Get all the Classes this Property belongs."""
        classes = []
        classesof = prp.get(f"{domain}/domainIncludes", [])
        for range in Crispr.list_anyway(classesof):
            classes.append(range["@id"])
        return classes

    @staticmethod
    def is_primitive_thing(cls, domain):
        """Check whether this Thing is a Primitive data type."""
        return f"{domain}/DataType" in Crispr.list_anyway(cls["@type"])

    @staticmethod
    def subclasses_direct(cls):
        if not cls:
            return []
        subclassesof = cls.get("rdfs:subClassOf", [])
        return [sub["@id"] for sub in Crispr.list_anyway(subclassesof)]

    @staticmethod
    def property_name(prp):
        """Parse all the different ways a property name is represented."""
        prp_name = prp["rdfs:label"]
        if isinstance(prp_name, dict):
            return prp_name["@value"]
        elif isinstance(prp_name, list):
            en = [p["@value"] for p in list if p["@language"] == "en"]
            if en:
                return en[0]
            else:
                [p["@value"] for p in list][0]
        else:
            return prp_name

    def name_from_url(self, url):
        return url.replace(self.domain + "/", "")

    def property_of_by_name(self, property_name):
        return self.property_of_by_url(f"{self.domain}/{property_name}")

    def class_of_by_name(self, thing_name):
        return self.class_of_by_url(f"{self.domain}/{thing_name}")

    def properties_of_by_name(self, thing_name):
        return self.properties_of_by_url(f"{self.domain}/{thing_name}")

    def enumerations_of_by_name(self, thing_name):
        return self.enumerations_of_by_url(f"{self.domain}/{thing_name}")

    def descendants_of_by_name(self, thing_name):
        return self.dependants_of_by_url(f"{self.domain}/{thing_name}")

    def dependencies_of_by_names(
        self, thing_names, max_depth=0, collated_things=[]
    ):
        return self.class_dependencies_of(
            [f"{self.domain}/{t}" for t in thing_names],
            max_depth,
            collated_things,
        )

    def property_of_by_url(self, property_url):
        """Return the dict of a property object from schema, given its URL."""
        for prp in self.properties:
            if Crispr.is_of(prp, property_url):
                return prp
        return None

    def class_of_by_url(self, thing_url):
        """Return the dict of a class object from schema, given its URL."""
        for cls in self.classes:
            if Crispr.is_of(cls, thing_url):
                return cls
        return None

    def properties_of_by_url(self, thing_url):
        """Return a dict list of a property objects of a class from schema."""
        properties = []
        for prp in self.properties:
            property_for = prp.get(f"{self.domain}/domainIncludes", [])
            for domain in Crispr.list_anyway(property_for):
                if Crispr.is_of(domain, thing_url):
                    properties.append(prp)
        return properties

    def enumerations_of_by_url(self, thing_url):
        """Return a dict list of a enumerations of a class from schema."""
        enumerations = []
        for enum in self.enumerated:
            if enum["@type"] == thing_url:
                enumerations.append(enum["rdfs:label"])
        return tuple((e, e) for e in enumerations)

    def property_enumerated(self, prp):
        """See if a property type is enumarated and return it."""
        property_types = Crispr.property_types_of(prp, self.domain)
        for thing_url in property_types:
            subclasses = self.subclasses_of(self.class_of_by_url(thing_url))
            if "http://schema.org/Enumeration" in subclasses:
                return thing_url
        return None

    def dependants_of_by_url(self, thing_url, max_level=1):
        """Get all the dependants of a class from schema."""
        dependants = []
        for cls in self.classes:
            if thing_url in Crispr.subclasses_direct(cls):
                dependant_url = cls["@id"]
                dependants.append(dependant_url)
                if max_level > 0:
                    more = self.dependants_of_by_url(
                        dependant_url, max_level - 1
                    )
                    if more:
                        dep = [self.name_from_url(d) for d in more]
                        # dependant_url = f"{dependant_url} {dep}"
                        dependants.append(dep)
        return dependants

    def subclasses_of(self, cls):
        subclasses = []
        for thing_url in Crispr.subclasses_direct(cls):
            Crispr.merge(subclasses, [thing_url])
            Crispr.merge(
                subclasses, self.subclasses_of(self.class_of_by_url(thing_url))
            )
        return subclasses

    def primitive_types_of(self, thing_urls):
        """Filter out types which aren't primitive."""
        primitives = []
        for thing_url in thing_urls:
            if thing_url in self.primitives:
                primitives.append(thing_url)
        return primitives

    def class_dependencies_of(
        self, thing_urls, max_depth=0, collated_things=[]
    ):
        """Returns a list of classes required for your desired things.

        To avoid having too many class types for properties, it will stop
        getting new property types after a certain depth.

        Recursive: In phases:

        - Phase1.1. Get all the ancestor classes required by the first set.
        - Phase1.2. Get all the properties required by all these classes.
        - Phase1.3. Get the classes of these properties.
        - Phase1.4. Pass any new found classes into Phase2.
        - Repeat for Phase2=>Phase(x).
        - Phase(x).3. Only get primitive classes after x>=max_depth Phases.

        Phase(x).3 insures that we don't need too many deeply nested ForeignKeys
        for Things we are collecting. At some point we'll just collect some Text
        or a Number for a property - which can be represented by a DB field
        type.

        Once you have finished, you can use this list of classes to build your
        models. If a certain "Thing" has a property type which is not listed by
        this function, you should swap it for a primitive type instead.
        """
        # Copy this phase's thing_urls as a starting point.
        phase_urls = [u for u in thing_urls]
        # Check each thing_url
        for thing_url in thing_urls:
            # Add the sub classes. This gives you the full model requirement
            # for this phase.
            ancestors = self.subclasses_of(self.class_of_by_url(thing_url))
            Crispr.merge(phase_urls, ancestors)
            # Get Property Types of thing + ancestors.
            for ancestor_url in ancestors + [thing_url]:
                # Check the Types of each thing's Properties.
                for prop in self.properties_of_by_url(ancestor_url):
                    property_types_of = Crispr.property_types_of(
                        prop, self.domain
                    )
                    # Prevent going any deeper.
                    if max_depth < 1:
                        # Remove non-primitive Types, if deep enough.
                        property_types_of = self.primitive_types_of(
                            property_types_of
                        )
                        """This greatly reduces the number of new Types."""
                    Crispr.merge(phase_urls, property_types_of)
        # Have we found new Types?
        newly_found = [u for u in phase_urls if u not in collated_things]
        if len(newly_found):
            # Start a new phase for just these new Types.
            return self.class_dependencies_of(
                newly_found,
                max_depth=max_depth - 1,
                collated_things=collated_things + newly_found,
            )
        else:
            # We've found them all!
            class_dependencies = Crispr.merge(collated_things, phase_urls)
            # Return all the non primitive things.
            return [
                t for t in class_dependencies if not t in self.primitives
            ]  # [t for t in class_dependencies if not t in self.primitives]



# -*- encoding: utf-8 -*-
from django.forms import ModelForm
from django.forms.widgets import (
    DateInput,
    DateTimeInput,
    HiddenInput,
    Select,
    TextInput,
    TimeInput,
)


class AnyEmailInput(TextInput):
    input_type = "email"


class AnyNumberInput(TextInput):
    input_type = "number"


class AnyTelephoneInput(TextInput):
    input_type = "tel"


class AnyDateInput(DateInput):
    input_type = "date"


class AnyDateTimeInput(DateTimeInput):
    input_type = "datetime-local"


class AnyTimeInput(TimeInput):
    input_type = "time"


class AnyForeignKeyInput(Select):
    input_type = "foreign"


def every_form(any_model, elio_key, page_link, field_dna=[]):
    class EveryForm(ModelForm):
        def __init__(self, *args, **kw):
            super().__init__(*args, **kw)
            for f, v in self.fields.items():
                if v.__class__.__name__ == "DateField":
                    self.fields[f].widget = AnyDateInput()
                elif v.__class__.__name__ == "TimeField":
                    self.fields[f].widget = AnyTimeInput()
                elif v.__class__.__name__ == "DateTimeField":
                    self.fields[f].widget = AnyDateTimeInput()
                elif v.__class__.__name__ == "EmailField":
                    self.fields[f].widget = AnyEmailInput()
                elif v.__class__.__name__ == "ModelChoiceField":
                    # self.fields[f].widget = AnyForeignKeyInput()
                    # print(
                    #     getattr(self.instance.__class__, f).field.related_model
                    # )
                    pass
                elif v.__class__.__name__ in [
                    "DurationField",
                    "FloatField",
                    "IntegerField",
                ]:
                    self.fields[f].widget = AnyNumberInput()
                elif v.__class__.__name__ in [
                    "CharField",
                    "URLField",
                    "BooleanField",
                    "ImageField",
                    "TypedChoiceField",
                ]:
                    pass
                else:
                    print("Needs widget?", v.__class__.__name__)
            self.fields["elio_key"].widget = HiddenInput()
            self.fields["elio_key"].initial = elio_key
            self.fields["elio_page"].widget = HiddenInput()
            self.fields["elio_page"].initial = page_link

        field_order = [
            "name",
            "alternateName",
            "disambiguatingDescription",
            "image",
            "description",
            "sameAs",
        ]

        class Meta:
            model = any_model
            exclude = ["pk", "polymorphic_ctype", "elio_role"]
            if field_dna:
                fields = field_dna + ["elio_key", "elio_page"]

    return EveryForm



# -*- encoding: utf-8 -*-
from django.urls import reverse_lazy


def get_page_dna(site_dna, current_page):
    """Find the details of the current page in the SITE_DNA."""
    for menu_key in site_dna.keys():
        if menu_key == current_page:
            return site_dna[menu_key]
    for menu_values in site_dna.values():
        if isinstance(menu_values, dict):
            rtn = get_page_dna(menu_values, current_page)
            if rtn:
                return rtn
    return {}


def get_model_dna(page_dna, thing_name):
    """Find the details of a model of the current page in the SITE_DNA."""
    if not isinstance(page_dna, dict):
        return {}
    for elio_type in ["engage", "list"]:
        if thing_name in list(page_dna.get(elio_type, {})):
            if isinstance(page_dna[elio_type], dict):
                return page_dna[elio_type].get(thing_name, {})
    return {}


def menuactivator(menu_dna, current_page):
    """A lookahead to return if a menu has sub menus activated."""
    for page_key in menu_dna.get("page", {}).keys():
        if page_key == current_page:
            return True
    for page_vals in menu_dna.get("page", {}).values():
        if page_vals.get("page"):
            return menuactivator(page_vals, current_page)
    return False


def get_page_menu(site_dna, current_page, site_things=[]):
    """Convert the SITE_DNA into a menu."""
    menu = dict()
    for page_key, page_vals in site_dna.get("page", {}).items():
        menu[page_key] = dict()
        menu[page_key]["url"] = reverse_lazy(
            "engage.page", kwargs={"page_key": page_key}
        )
        if page_vals.get("status"):
            menu[page_key]["status"] = page_vals.get("status")
        if page_key == current_page:
            menu[page_key]["status"] = "active"
        if menu[page_key].get("status") or menuactivator(
            page_vals, current_page
        ):
            if not menu[page_key].get("status"):
                menu[page_key]["status"] = "crumb"  # Make the breadcrumbs
            sub_menu = get_page_menu(page_vals, current_page)
            if sub_menu.get("page"):
                menu[page_key].update(sub_menu)
    if site_things:
        menu["Browse Things"] = {"status": "label", "page": {}}
        for thing in sorted(site_things):
            menu["Browse Things"]["page"][thing] = dict()
            menu["Browse Things"]["page"][thing]["url"] = reverse_lazy(
                "list.things", kwargs={"page_key": thing, "thing": thing}
            )
            if thing == current_page:
                menu["Browse Things"]["page"][thing]["status"] = "active"
    return {"page": menu}



"""**Usage**

::

    # Show the full dna in advance
    django-admin dna Thing Person -d 0"""
# -*- encoding: utf-8 -*-
import json
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from dna.dna import DnaOfDjangoModel


class Command(BaseCommand):
    """Build the dna of models you can use to build django Models from."""

    def add_arguments(self, parser):
        """
        :param name: things.
        """
        parser.add_argument(
            "things", nargs="+", type=str, help="Space delimited list of things"
        )
        parser.add_argument(
            "--depth",
            "-d",
            type=int,
            help="Max depth to accept object types for properties.",
        )
        parser.add_argument(
            "--path", "-p", type=str, help="Path of the schema's jsonld."
        )

    def handle(self, *args, **options):
        names = options["things"]
        depth = options["depth"]
        schema_path = options["path"]
        dna = DnaOfDjangoModel(schema_path, app_label="dna", no_comments=True)
        dna._get_replicate_dna(names, depth)
        with open("dna.py", "w") as f:
            f.write(json.dumps(dna.dna_loaded, sort_keys=True, indent=4))



# -*- encoding: utf-8 -*-
from django.conf import settings
from dna.dna import DnaOfDjangoModel
from dna.models import every_model


every_model(
    DnaOfDjangoModel(
        settings.SCHEMA_PATH, settings.SITE_DNA, settings.DNA_DEPTH
    ).build()
)



"""ReplicateDjangoModel: Return django models.

This class builds dynamic Django models the DNA of schema.org "Things".

Initialize with the product of `DnaOfDjangoModel`.`_get_replicate_dna`. The
`start_replication` method will build each Django model which will be registered
as if hand coded in a models.py file. You will be able to run `makemigrations`
and  `migrate` as usual.


**Usage**

::

    from dna.dna import DnaOfDjangoModel
    from dna.replicate import ReplicateDjangoModel

    # First build or retrieve from storage the DNA of models.
    dna_maker = DnaOfDjangoModel("schema.jsonld", "app")
    dna = dna_maker._get_replicate_dna(<SITE_DNA>, 1)

    # Then instantiate this class and start the model registration.
    replicator = ReplicateDjangoModel(dna, "app")
    replicator.start_replication()"""
# -*- encoding: utf-8 -*
from django.conf import settings
from django.db import models
from django.urls import resolve, reverse
from polymorphic.models import PolymorphicModel


class ReplicateDjangoModel:
    """Build django models from the DNA."""

    def __init__(self, model_dna):
        # Schema.org jsonld information as parsable units.
        self.dna = model_dna
        # Holding places for the structure `ReplicateDjangoModel` builds.
        self.models_loaded = {"models.Model": PolymorphicModel}

    def get_model_field_dynamically(self, dna_of_field):
        """Returns an instance of a django model type."""
        field_type = dna_of_field.pop("field_type")
        if field_type == "ForeignKey":
            related_to = dna_of_field.pop("related_to")
            dna_of_field["on_delete"] = models.CASCADE
            return models.ForeignKey(related_to, **dna_of_field)
        else:
            django_model_type = getattr(models, field_type)
            return django_model_type(**dna_of_field)

    def get_model_inheritance(self, inherit_from):
        """Get or create a model required for inheritance."""
        model_inheritance = self.models_loaded.get(inherit_from, None)
        if not model_inheritance:
            model_inheritance = self.replicate_django_model(
                inherit_from, self.dna[inherit_from]
            )
        return model_inheritance

    def replicate_django_model(self, thing_name, dna_of_model, options=None):
        """Return a django model given a class object.

        :param dna_of_model: e.g.
            {
                "fields_of": {
                    "name": {
                        "field_type": "CharField",
                        "max_length": 512,
                        "blank": true,
                    },
                    "identifier": {
                        "blank": true,
                        "field_type": "ForeignKey",
                        "related_name": "dnathing_dnacreativework_about",
                        "related_to": "Thing",
                        "null": true,
                    },
                    "description": {"field_type": "TextField", "blank": true},
                },
                "inherits_from": ["base.Model"],
                "thing_name": "AppnameThing",
            }
        """
        # Don't keep getting the same model over and over.
        model = self.models_loaded.get(thing_name, None)
        if model:
            return model

        class Meta:
            db_table = thing_name
            app_label = settings.APP_NAME

        def get_absolute_url(self):
            return self.elio_page

        def get_fields(self):
            return [
                (field.name, type(field).__name__, getattr(self, field.name))
                for field in self._meta.fields
                if type(field).__name__ not in ["ImageField", "FileField"]
            ]

        def get_images(self):
            # Get all the image fields
            images = [
                (field, self)
                for field in self._meta.fields
                if type(field).__name__ in ["ImageField"]
            ]
            # Return only images with images.
            return [
                (i[0].name, i[1].image.url, i[1].image.height, i[1].image.width)
                for i in images
                if i[1].image
            ]

        def __str__(self):
            return self.name

        # Update Meta with any options that were provided
        if options is not None:
            for key, value in options.iteritems():
                setattr(Meta, key, value)

        # Set up a dictionary to simulate declarations within a class
        attrs = {
            "__module__": __name__,
            "__str__": __str__,
            "Meta": Meta,
            "get_absolute_url": get_absolute_url,
            "get_fields": get_fields,
            "get_images": get_images,
        }
        # Model Inheritance.
        inheritance = []
        for inherit_from in dna_of_model["inherits_from"]:
            model_inheritance = self.get_model_inheritance(inherit_from)
            inheritance.append(model_inheritance)

        # Fields.
        for field_name, dna_of_field in dna_of_model["fields_of"].items():
            attrs[field_name] = self.get_model_field_dynamically(dna_of_field)
        # Add some additional system fields.
        if thing_name == "Thing":
            attrs["elio_key"] = models.SlugField(max_length=128, unique=False)
            attrs["elio_page"] = models.CharField(
                blank=True, max_length=128, null=True
            )
            attrs["elio_role"] = models.CharField(
                blank=False, max_length=8, null=False, default="list"
            )
        # Manually create the inheritance pointer.
        for point_to in inheritance:
            if not point_to.__name__ == "PolymorphicModel":
                attrs[
                    f"{thing_name}_to_{point_to.__name__}".lower()
                ] = models.OneToOneField(
                    f"{point_to.__name__}",
                    parent_link=True,
                    related_name=f"{point_to.__name__}_for_{thing_name}",
                    on_delete=models.CASCADE,
                )
        # Create the class, which automatically triggers ModelBase processing.
        model = type(thing_name, tuple(inheritance), attrs)
        # Store it so that we can pick it up for inheritance.
        self.models_loaded[thing_name] = model
        # Return the model.
        return model

    def start_replication(self):
        """Build all the models."""
        # Do each model in any order
        for thing_name, model_dna in self.dna.items():
            self.replicate_django_model(thing_name, model_dna)



# -*- encoding: utf-8 -*-
from dna.views import (
    EveryCreateViewMixin,
    EveryUpdateViewMixin,
    EveryDeleteViewMixin,
    EveryEngageViewMixin,
    EveryListViewMixin,
    EveryIterateViewMixin,
    EveryOptimizeViewMixin,
    EveryPageViewMixin,
    EveryHomeViewMixin,
)


class Criew(EveryCreateViewMixin):
    template_name = "chromosomes/thing_form.html"


class Uiew(EveryUpdateViewMixin):
    template_name = "chromosomes/thing_form.html"


class Diew(EveryDeleteViewMixin):
    template_name = "chromosomes/thing_delete.html"


class Eiew(EveryEngageViewMixin):
    template_name = "chromosomes/thing_engaged.html"


class Liew(EveryListViewMixin):
    template_name = "chromosomes/thing_listed.html"


class Iew(EveryIterateViewMixin):
    template_name = "chromosomes/thing_iterate.html"


class Oiew(EveryOptimizeViewMixin):
    template_name = "chromosomes/thing_optimize.html"


class Piew(EveryPageViewMixin):
    template_name = "chromosomes/thing_page.html"


class Hiew(EveryHomeViewMixin):
    template_name = "chromosomes/home_page.html"



import os
from django.conf import settings
from easy_thumbnails.files import get_thumbnailer


self_dir = os.path.dirname(os.path.realpath(__file__))
app_path = os.sep.join(self_dir.split(os.sep)[:-1])


def watermark(thing_image, dimension):
    thumbnail_options = settings.THUMBNAIL_ALIASES[""][dimension]
    thumber = get_thumbnailer(thing_image)
    thumb = thumber.get_thumbnail(thumbnail_options)
    return thumb.url



# -*- encoding: utf-8 -*-
from django.conf import settings
from django.conf.urls.static import static
from django.urls import path

from .views import Criew, Uiew, Diew, Eiew, Liew, Iew, Oiew, Piew, Hiew


urlpatterns = [
    path("cr/<str:page_key>/<str:thing>", Criew.as_view(), name="create.thing"),
    path(
        "u/<str:page_key>/<str:thing>/<int:pk>",
        Uiew.as_view(),
        name="update.thing",
    ),
    path(
        "d/<str:page_key>/<str:thing>/<int:pk>",
        Diew.as_view(),
        name="delete.thing",
    ),
    path(
        "e/<str:page_key>/<str:thing>/<int:pk>",
        Eiew.as_view(),
        name="engage.thing",
    ),
    path("l/<str:page_key>/<str:thing>", Liew.as_view(), name="list.things"),
    path(
        "i/<str:page_key>/<str:thing>/<int:pk>/<str:field_name>",
        Iew.as_view(),
        name="iterate.things",
    ),
    path(
        "o/<str:page_key>/<str:thing>", Oiew.as_view(), name="optimize.things"
    ),
    path("p/<str:page_key>", Piew.as_view(), name="engage.page"),
    path("", Hiew.as_view(), name="home"),
]


if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
